package com.mx.dla.dda.contrato.titulo.daos;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import com.mx.dla.dda.contrato.titulo.dtos.Lista;

public interface ListaDAO {

	void insertarLista(Lista lista);
	void actualizarLista(Lista lista);
	void borrarLista(@Param("idLista") Long idLista, @Param("prefix") String prefix );
	void borrarListaPorIdContrato(@Param("idContrato") Long idContrato, @Param("prefix") String prefix );
	List<Lista> buscarListas(@Param("idContrato") Long idContrato , @Param("prefix") String prefix);
	Lista buscarLista(@Param("idLista") Long idLista , @Param("prefix") String prefix);
	Lista buscarListaActual(@Param("idContrato") Long idContrato , @Param("prefix") String prefix);
	List<Lista> buscarListasPorContratoCreacion(@Param("idContratoCreacion") Long idContratoCreacion , @Param("prefix") String prefix);
	void updateListaTitulos(Lista lista);
}

